/*******************************************************************************

  Morphology for DIMIN Viewer n5 (DIV5)

  Programmer: Dima V. Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  History:
    06/01/2005 20:09 - First Creation

  ver: 1

*******************************************************************************/

#include <windows.h>

#include "div5_morphology.h"

TDIVPluginHeader PluginHeader = {
  sizeof( TDIVPluginHeader ),
  5,   // plugin interface version - MUST BE 5
  DIV_PLUGIN_TYPE_FILTER,
  1,   // plugin version
  11,  // 8 filters + 3 dividers
  "DIMIN Morphology"
};

//------------------------------------------------------------------------------
// About procedure
//------------------------------------------------------------------------------
void __stdcall aboutPlugin( HWND hwndParent, HINSTANCE hDllInstance ) 
{
  MessageBox(GetFocus(), "Morphology Filters v1", "About Plug-in", MB_OK | MB_ICONEXCLAMATION);
}



//------------------------------------------------------------------------------
// Filter 3 find edges
//------------------------------------------------------------------------------

TDIVTagedArgument amountFilterArgsArr = {
  1000,         // id number, plugin may use ids above 1000 as private
  DIM_TAG_LONG, // TDimTagType
  1,            // number of records of type tagType pointed by tagData
  "Amount:",    // text description of the argument
  (void *) 1,   // pointer to data, if data size is <= 4 bytes
                // then it's directly stored into pointer
  (void *) 1,   // pointer to min val, used if needed to define
                // a special value retrieval, e.g. -100 .. 100
  (void *) 50   // pointer to max val
};

TDIVArguments filter01Args = {
  1,          // 0 - no preview window, 1 - DIV will open preview
  1,          // number of arguments in the array
  &amountFilterArgsArr, // pointer to array of argumants
  NULL, // custom controls creation INIT
  NULL, // custom controls creation QUIT
};

TDIVFilterHeader filterHeader01 = {
  sizeof(TDIVFilterHeader),
  "DIV_MORPHOLOGY_DILATION_SQUARE",
  "Dilation Square",
  filterDilationSquare3x3, // filter
  666,       // no bpp needed
  &filter01Args,    // arguments, if NULL equals to usePreview = 0
  aboutPlugin
};

TDIVFilterHeader filterHeader02 = {
  sizeof(TDIVFilterHeader),
  "DIV_MORPHOLOGY_DILATION_CROSS",
  "Dilation Cross",
  filterDilationCross3x3, // filter
  666,       // no bpp needed
  &filter01Args,    // arguments, if NULL equals to usePreview = 0
  aboutPlugin
};

TDIVFilterHeader filterHeader03 = {
  sizeof(TDIVFilterHeader),
  "DIV_MORPHOLOGY_EROSION_SQUARE",
  "Erosion Square",
  filterErosionSquare3x3, // filter
  666,       // no bpp needed
  &filter01Args,    // arguments, if NULL equals to usePreview = 0
  aboutPlugin
};

TDIVFilterHeader filterHeader04 = {
  sizeof(TDIVFilterHeader),
  "DIV_MORPHOLOGY_EROSION_CROSS",
  "Erosion Cross",
  filterErosionCross3x3, // filter
  666,       // no bpp needed
  &filter01Args,    // arguments, if NULL equals to usePreview = 0
  aboutPlugin
};


TDIVFilterHeader filterHeader05 = {
  sizeof(TDIVFilterHeader),
  "DIV_MORPHOLOGY_OPENING_SQUARE",
  "Opening Square",
  filterOpeningSquare3x3, // filter
  666,       // no bpp needed
  &filter01Args,    // arguments, if NULL equals to usePreview = 0
  aboutPlugin
};

TDIVFilterHeader filterHeader06 = {
  sizeof(TDIVFilterHeader),
  "DIV_MORPHOLOGY_OPENING_CROSS",
  "Opening Cross",
  filterOpeningCross3x3, // filter
  666,       // no bpp needed
  &filter01Args,    // arguments, if NULL equals to usePreview = 0
  aboutPlugin
};

TDIVFilterHeader filterHeader07 = {
  sizeof(TDIVFilterHeader),
  "DIV_MORPHOLOGY_CLOSING_SQUARE",
  "Closing Square",
  filterClosingSquare3x3, // filter
  666,       // no bpp needed
  &filter01Args,    // arguments, if NULL equals to usePreview = 0
  aboutPlugin
};

TDIVFilterHeader filterHeader08 = {
  sizeof(TDIVFilterHeader),
  "DIV_MORPHOLOGY_CLOSING_CROSS",
  "Closing Cross",
  filterClosingCross3x3, // filter
  666,       // no bpp needed
  &filter01Args,    // arguments, if NULL equals to usePreview = 0
  aboutPlugin
};

TDIVFilterHeader headerDevider = {
  sizeof(TDIVFilterHeader),
  "-",
  "-",
  NULL, // filter
  0,       // no bpp needed
  0,    // arguments, if NULL equals to usePreview = 0
  NULL
};

//***************************************************************************
// DLL entrie
//***************************************************************************

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
  switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}


TDIVPluginHeader* __stdcall divGetPluginHeader()
{
   return &PluginHeader;
}

TDIVFilterHeader* __stdcall divGetFilterHeader(int I)
{
  switch (I)
  {
    case 0: return &filterHeader01;
    case 1: return &filterHeader02;
    
    case 2: return &headerDevider;

    case 3: return &filterHeader03;
    case 4: return &filterHeader04;

    case 5: return &headerDevider;

    case 6: return &filterHeader05;
    case 7: return &filterHeader06;

    case 8: return &headerDevider;

    case 9: return &filterHeader07;
    case 10: return &filterHeader08;
    default : return NULL;
  }
}

TDivPluginFilter __stdcall divGetFilter(int I)
{
  switch (I)
  {
    case 0: return filterDilationSquare3x3;
    case 1: return filterDilationCross3x3;

    case 2: return NULL;

    case 3: return filterErosionSquare3x3;
    case 4: return filterErosionCross3x3;
    
    case 5: return NULL;

    case 6: return filterOpeningSquare3x3;
    case 7: return filterOpeningCross3x3;

    case 8: return NULL;

    case 9: return filterClosingSquare3x3;
    case 10: return filterClosingCross3x3;
    default: return NULL;
  }
}

//---------------------------------------------------------------------------
