/*******************************************************************************

  Morphology for DIMIN Viewer n5 (DIV5)

  Programmer: Dima V. Fedorov Levit <dimin@dimin.net> <http://www.dimin.net/>

  History:
    06/01/2005 22:17 - First Creation
  
  ver: 1  
*******************************************************************************/

#include <stdio.h>
#include <windows.h>

#include "DIVPluginDefs.h"
#include "DIVFilterDefs.h"
#include "DIVFilterUtils.h"


//****************************************************************************
// DILATION
//****************************************************************************

void dilateSquare33( TDIVParameters *par, BITMAP *inbmp, int amount )
{
  unsigned char *p0, *p1, *p2, *pr;
  long x,y;

  unsigned int h=inbmp->bmHeight, w=inbmp->bmWidth;
  uint bpp = inbmp->bmBitsPixel * inbmp->bmPlanes;
  unsigned char *inp = (unsigned char *) inbmp->bmBits;

  DIM_UCHAR *tp;
  HPALETTE hpal;
  HBITMAP temp = dimCreateHBitmap( w, h, bpp, 0, NULL, &tp, hpal );
  
  int i;
  for (i=0; i<amount; i++)
  {
    uint pB=0, pE=w-1;
    uint pI=0;
    memcpy( tp, inp, getHBLineSizeInBytes(w, bpp) * h );
  
    if (bpp == 8)
    for (y=1; y<h-1; y++)
    {
      p1 = (unsigned char *) getHBScanLine( h-y-0, tp, w, bpp );
      p0 = (unsigned char *) getHBScanLine( h-y-1, tp, w, bpp );
      p2 = (unsigned char *) getHBScanLine( h-y-2, tp, w, bpp );
      pr = (unsigned char *) getHBScanLine( h-y-1, inp, w, bpp );

      showProgress( par, y, h, "Dilation Square" );
      if ( testAbort( par ) == 1) break;
      findLineBegEndTopBot( par, w, y, pI, pB, pE );

      if (pE>pB)
      for (x=pB+1; x<=pE-1; x++)
        pr[x] = max( max( max( max( p1[x-1], p1[x] ), p1[x+1] ),
                          max( max( p0[x-1], p0[x] ), p0[x+1] ) ),
                     max( max( p2[x-1], p2[x] ), p2[x+1] ) );
    } // for y 8 bpp

    if (bpp == 24)
    for (y=1; y<h-1; y++)
    {
      p1 = (unsigned char *) getHBScanLine( h-y-0, tp, w, bpp );
      p0 = (unsigned char *) getHBScanLine( h-y-1, tp, w, bpp );
      p2 = (unsigned char *) getHBScanLine( h-y-2, tp, w, bpp );
      pr = (unsigned char *) getHBScanLine( h-y-1, inp, w, bpp );

      showProgress( par, y, h, "Dilation Square" );
      if ( testAbort( par ) == 1) break;
      findLineBegEndTopBot( par, w, y, pI, pB, pE );

      if (pE>pB)
      for (x=(pB+1)*3; x<pE*3; x++)
        pr[x] = max( max( max( max( p1[x-3], p1[x] ), p1[x+3] ),
                          max( max( p0[x-3], p0[x] ), p0[x+3] ) ),
                     max( max( p2[x-3], p2[x] ), p2[x+3] ) );
    } // for y 24 bpp

  } // for amount

  DeleteObject( temp );
}

void dilateCross33( TDIVParameters *par, BITMAP *inbmp, int amount )
{
  unsigned char *p0, *p1, *p2, *pr;
  long x,y;

  unsigned int h=inbmp->bmHeight, w=inbmp->bmWidth;
  uint bpp = inbmp->bmBitsPixel * inbmp->bmPlanes;
  unsigned char *inp = (unsigned char *) inbmp->bmBits;

  DIM_UCHAR *tp;
  HPALETTE hpal;
  HBITMAP temp = dimCreateHBitmap( w, h, bpp, 0, NULL, &tp, hpal );
  
  int i;
  for (i=0; i<amount; i++)
  {
    uint pB=0, pE=w-1;
    uint pI=0;
    memcpy( tp, inp, getHBLineSizeInBytes(w, bpp) * h );
  
    if (bpp == 8)
    for (y=1; y<h-1; y++)
    {
      p1 = (unsigned char *) getHBScanLine( h-y-0, tp, w, bpp );
      p0 = (unsigned char *) getHBScanLine( h-y-1, tp, w, bpp );
      p2 = (unsigned char *) getHBScanLine( h-y-2, tp, w, bpp );
      pr = (unsigned char *) getHBScanLine( h-y-1, inp, w, bpp );

      showProgress( par, y, h, "Dilation Cross" );
      if ( testAbort( par ) == 1) break;
      findLineBegEndTopBot( par, w, y, pI, pB, pE );

      if (pE>pB)
      for (x=pB+1; x<=pE-1; x++)
        pr[x] = max( max( max( max( p1[x], p0[x-1]), p0[x] ), p0[x+1] ), p2[x] );
    } // for y 8 bpp

    if (bpp == 24)
    for (y=1; y<h-1; y++)
    {
      p1 = (unsigned char *) getHBScanLine( h-y-0, tp, w, bpp );
      p0 = (unsigned char *) getHBScanLine( h-y-1, tp, w, bpp );
      p2 = (unsigned char *) getHBScanLine( h-y-2, tp, w, bpp );
      pr = (unsigned char *) getHBScanLine( h-y-1, inp, w, bpp );

      showProgress( par, y, h, "Dilation Cross" );
      if ( testAbort( par ) == 1) break;
      findLineBegEndTopBot( par, w, y, pI, pB, pE );

      if (pE>pB)
      for (x=(pB+1)*3; x<pE*3; x++)
        pr[x] = max( max( max( max( p1[x], p0[x-3]), p0[x] ), p0[x+3] ), p2[x] );
    } // for y 24 bpp

  } // for amount

  DeleteObject( temp );
}

//****************************************************************************
// EROSION
//****************************************************************************

void erodeSquare33( TDIVParameters *par, BITMAP *inbmp, int amount )
{
  unsigned char *p0, *p1, *p2, *pr;
  long x,y;

  unsigned int h=inbmp->bmHeight, w=inbmp->bmWidth;
  uint bpp = inbmp->bmBitsPixel * inbmp->bmPlanes;
  unsigned char *inp = (unsigned char *) inbmp->bmBits;

  DIM_UCHAR *tp;
  HPALETTE hpal;
  HBITMAP temp = dimCreateHBitmap( w, h, bpp, 0, NULL, &tp, hpal );
  
  int i;
  for (i=0; i<amount; i++)
  {
    uint pB=0, pE=w-1;
    uint pI=0;
    memcpy( tp, inp, getHBLineSizeInBytes(w, bpp) * h );
  
    if (bpp == 8)
    for (y=1; y<h-1; y++)
    {
      p1 = (unsigned char *) getHBScanLine( h-y-0, tp, w, bpp );
      p0 = (unsigned char *) getHBScanLine( h-y-1, tp, w, bpp );
      p2 = (unsigned char *) getHBScanLine( h-y-2, tp, w, bpp );
      pr = (unsigned char *) getHBScanLine( h-y-1, inp, w, bpp );

      showProgress( par, y, h, "Erosion Square" );
      if ( testAbort( par ) == 1) break;
      findLineBegEndTopBot( par, w, y, pI, pB, pE );

      if (pE>pB)
      for (x=pB+1; x<=pE-1; x++)
        pr[x] = min( min( min( min( p1[x-1], p1[x] ), p1[x+1] ),
                          min( min( p0[x-1], p0[x] ), p0[x+1] ) ),
                     min( min( p2[x-1], p2[x] ), p2[x+1] ) );
    } // for y 8 bpp

    if (bpp == 24)
    for (y=1; y<h-1; y++)
    {
      p1 = (unsigned char *) getHBScanLine( h-y-0, tp, w, bpp );
      p0 = (unsigned char *) getHBScanLine( h-y-1, tp, w, bpp );
      p2 = (unsigned char *) getHBScanLine( h-y-2, tp, w, bpp );
      pr = (unsigned char *) getHBScanLine( h-y-1, inp, w, bpp );

      showProgress( par, y, h, "Erosion Square" );
      if ( testAbort( par ) == 1) break;
      findLineBegEndTopBot( par, w, y, pI, pB, pE );

      if (pE>pB)
      for (x=(pB+1)*3; x<pE*3; x++)
        pr[x] = min( min( min( min( p1[x-3], p1[x] ), p1[x+3] ),
                          min( min( p0[x-3], p0[x] ), p0[x+3] ) ),
                     min( min( p2[x-3], p2[x] ), p2[x+3] ) );
    } // for y 24 bpp

  } // for amount

  DeleteObject( temp );
}

void erodeCross33( TDIVParameters *par, BITMAP *inbmp, int amount )
{
  unsigned char *p0, *p1, *p2, *pr;
  long x,y;

  unsigned int h=inbmp->bmHeight, w=inbmp->bmWidth;
  uint bpp = inbmp->bmBitsPixel * inbmp->bmPlanes;
  unsigned char *inp = (unsigned char *) inbmp->bmBits;

  DIM_UCHAR *tp;
  HPALETTE hpal;
  HBITMAP temp = dimCreateHBitmap( w, h, bpp, 0, NULL, &tp, hpal );
  
  int i;
  for (i=0; i<amount; i++)
  {
    uint pB=0, pE=w-1;
    uint pI=0;
    memcpy( tp, inp, getHBLineSizeInBytes(w, bpp) * h );
  
    if (bpp == 8)
    for (y=1; y<h-1; y++)
    {
      p1 = (unsigned char *) getHBScanLine( h-y-0, tp, w, bpp );
      p0 = (unsigned char *) getHBScanLine( h-y-1, tp, w, bpp );
      p2 = (unsigned char *) getHBScanLine( h-y-2, tp, w, bpp );
      pr = (unsigned char *) getHBScanLine( h-y-1, inp, w, bpp );

      showProgress( par, y, h, "Erosion Cross" );
      if ( testAbort( par ) == 1) break;
      findLineBegEndTopBot( par, w, y, pI, pB, pE );

      if (pE>pB)
      for (x=pB+1; x<=pE-1; x++)
        pr[x] = min( min( min( min( p1[x], p0[x-1]), p0[x] ), p0[x+1] ), p2[x] );
    } // for y 8 bpp

    if (bpp == 24)
    for (y=1; y<h-1; y++)
    {
      p1 = (unsigned char *) getHBScanLine( h-y-0, tp, w, bpp );
      p0 = (unsigned char *) getHBScanLine( h-y-1, tp, w, bpp );
      p2 = (unsigned char *) getHBScanLine( h-y-2, tp, w, bpp );
      pr = (unsigned char *) getHBScanLine( h-y-1, inp, w, bpp );

      showProgress( par, y, h, "Erosion Cross" );
      if ( testAbort( par ) == 1) break;
      findLineBegEndTopBot( par, w, y, pI, pB, pE );

      if (pE>pB)
      for (x=(pB+1)*3; x<pE*3; x++)
        pr[x] = min( min( min( min( p1[x], p0[x-3]), p0[x] ), p0[x+3] ), p2[x] );
    } // for y 24 bpp

  } // for amount

  DeleteObject( temp );
}

//****************************************************************************
// OPENING
//****************************************************************************

void openSquare33( TDIVParameters *par, BITMAP *inbmp, int amount )
{
  erodeSquare33( par, inbmp, amount );
  dilateSquare33( par, inbmp, amount );
}

void openCross33( TDIVParameters *par, BITMAP *inbmp, int amount )
{
  erodeCross33( par, inbmp, amount );
  dilateCross33( par, inbmp, amount );
}

//****************************************************************************
// CLOSING
//****************************************************************************

void closeSquare33( TDIVParameters *par, BITMAP *inbmp, int amount )
{
  dilateSquare33( par, inbmp, amount );
  erodeSquare33( par, inbmp, amount );
}

void closeCross33( TDIVParameters *par, BITMAP *inbmp, int amount )
{
  dilateCross33( par, inbmp, amount );
  erodeCross33( par, inbmp, amount );
}
